from matchescu.matching.attribute._match import (
    BinarySimilarityMatch,
    BinarySimilarityMatchOnThreshold,
    RawMatch,
    SimilarityMatch,
    TernarySimilarityMatch,
    TernarySimilarityMatchOnThreshold,
)
from matchescu.matching.attribute._match_result import (
    BinaryResult,
    TernaryResult,
)
from matchescu.matching.attribute._typing import AttrMatchCallable

__all__ = [
    "AttrMatchCallable",
    "BinaryResult",
    "BinarySimilarityMatch",
    "BinarySimilarityMatchOnThreshold",
    "RawMatch",
    "SimilarityMatch",
    "TernaryResult",
    "TernarySimilarityMatch",
    "TernarySimilarityMatchOnThreshold",
]
